import uvicore
from uvicore.support.dumper import dump, dd
from uvicore.console import command, argument, option


@command()
async def cli():
    """xx_AppName xx_name CLI command"""

    print('Welcome to your new xx_AppName xx_name CLI command!')
